@@ -109,11 +109,13 @@ fitme <- function(formula,data, ## matches minimal call of HLfit
   assign("spaMM_glm_conv_crit",list(max=-Inf) , envir=environment(spaMM_glm.fit))
   time1 <- Sys.time()
   oricall <- match.call(expand.dots=TRUE) ## mc including dotlist
+  oricall$control.HLfit <- eval(oricall$control.HLfit, parent.frame()) # to evaluate variables in the formula_env, otherwise there are bugs in waiting 
   mc <- oricall
   mc[[1L]] <- get(".preprocess_formula", asNamespace("spaMM"), inherits=FALSE)  ## https://stackoverflow.com/questions/10022436/do-call-in-combination-with
   oricall$formula <- mc$formula <- eval(mc,parent.frame()) # 
   ## : among other effects, forces eval of promise for formula, so re-evaluating the call later will work 
-  ## [cf probitgem re-evaluating fitme(form,.....) in eval_smoothtest()]
+  ## [cf probitgem re-evaluating fitme(form,.....) in eval_smoothtest()];
+  ## likewise, associates the evaluated formula_env to the formula
   mc[[1L]] <- get(".check_args_fitme", asNamespace("spaMM"), inherits=FALSE) 
   mc <- eval(mc,parent.frame()) # 
   mc[[1L]] <- get(".preprocess_fitme", asNamespace("spaMM"), inherits=FALSE)
