@@ -16,35 +16,84 @@ def_sXaug_EigenDense_QRP_Chol_scaled <- function(Xaug, # already ZAL_scaled
   return( Xaug ) 
 }
 
-
+# trace(get_from_MME, print=FALSE, tracer=quote(cat("'",crayon::yellow(which),"'")))
+# trace(spaMM:::.sXaug_EigenDense_QRP_Chol_scaled, print=FALSE, tracer=quote(cat("'",which,"'")))
+#
 .sXaug_EigenDense_QRP_Chol_scaled <- function(sXaug,which="",szAug=NULL,B=NULL) {
   BLOB <- attr(sXaug,"BLOB") ## an environment
   if (is.null(BLOB$R_scaled)) {
+    delayedAssign("seq_n_u_h", seq_len(attr(sXaug,"n_u_h")), assign.env = BLOB )
+    delayedAssign("invsqrtwranef", 1/sqrt(attr(sXaug,"w.ranef")), assign.env = BLOB )
+    delayedAssign("t_Q_scaled", {
+      if ( ! is.null(BLOB$perm)) {
+        backsolve(BLOB$R_scaled,t(sXaug[,BLOB$perm]),transpose=TRUE)
+      } else backsolve(BLOB$R_scaled,t(sXaug),transpose=TRUE)}, assign.env = BLOB )  ## but repetitive usage is minimal?
+    delayedAssign("inv_d2hdv2", {
+      if (.is_evaluated("inv_factor_wd2hdv2w", BLOB)) { 
+        inv_d2hdv2 <- .crossprod(BLOB$inv_factor_wd2hdv2w) # inv_factor_wd2hdv2w is crossfactor
+      } else inv_d2hdv2 <- chol2inv(BLOB$R_R_v)
+      inv_d2hdv2 <- .m_Matrix_times_Dvec(inv_d2hdv2, BLOB$invsqrtwranef) ## _m_atrix sweep 2L
+      inv_d2hdv2 <- - .Dvec_times_matrix(BLOB$invsqrtwranef,inv_d2hdv2)
+    }, assign.env = BLOB )
+    delayedAssign("R_R_v", { # in "absdiag_R_v", "solve_d2hdv2","hatval_Z","R_scaled_v_h_blob", "logdet_R_scaled_v","Mg_invH_g"
+      seq_n_u_h <- BLOB$seq_n_u_h
+      # remove $u_h_cols_on_left code in version 2.1.61 since there is a bug in it: further code may require 
+      #  no nontrivial perm_R_v, so we remove perm_R_v from further code (as in Matrix_QRP_CHM_scaled version)
+      if ( is.null(BLOB$sortPerm)) { # default
+        R_R_v <- BLOB$R_scaled[seq_n_u_h,seq_n_u_h, drop=FALSE]
+      } else {
+        wd2hdv2w <- .crossprodCpp_d(BLOB$R_scaled[,BLOB$sortPerm[ seq_n_u_h ], drop=FALSE], NULL)
+        R_R_v <- .Rcpp_chol_R(wd2hdv2w)$R
+      }
+    }, assign.env = BLOB )
+    delayedAssign("absdiag_R_v",if ( is.null(BLOB$sortPerm)) { # default
+      abs(.diagfast(x=BLOB$R_scaled)[BLOB$seq_n_u_h])
+    } else { # non-default, but to optimize this, perhaps try to get the 
+      #  determinant(BLOB$R_scaled[,BLOB$sortPerm[ seq_n_u_h ], drop=FALSE])
+      # rather than go through the chol crossprod stuff ? 
+      abs(.diagfast(x=BLOB$R_R_v))
+    } , assign.env = BLOB )
+    delayedAssign("inv_factor_wd2hdv2w", {.Rcpp_backsolve(BLOB$R_R_v, NULL, # NULL i.e., identity, but without creating it (but this does not explain most of the time gain) in Rcpp
+                                     transpose=TRUE)}, assign.env = BLOB )
+    delayedAssign("logdet_R_scaled_b_v", sum(log(abs(.diagfast(x=BLOB$R_scaled)))), assign.env = BLOB )
+    delayedAssign("logdet_R_scaled_v", sum(log(BLOB$absdiag_R_v)), assign.env = BLOB )
+    delayedAssign("logdet_r22", {
+      # the R's are H-scaled but r22 is H-unscaled... tricky!
+      if ( is.null(BLOB$sortPerm)) { # default
+        sum(log(abs(.diagfast(x=BLOB$R_scaled)[-BLOB$seq_n_u_h]))) - attr(sXaug,"pforpv")*log(attr(sXaug,"H_global_scale"))/2 # '-' not '<-'
+      } else {
+        BLOB$logdet_R_scaled_b_v - BLOB$logdet_R_scaled_v - attr(sXaug,"pforpv")*log(attr(sXaug,"H_global_scale"))/2 # '-' not '<-'
+      }
+    } , assign.env = BLOB )
+    delayedAssign("logdet_sqrt_d2hdv2", { sum(log( sqrt(attr(sXaug,"w.ranef"))*BLOB$absdiag_R_v )) } , assign.env = BLOB )
+    delayedAssign("hatval", {colSums(BLOB$t_Q_scaled^2)} , assign.env = BLOB ) # also named hatval_ZX
+    ##############################
     EigenDense_QRP_method <- .spaMM.data$options$EigenDense_QRP_method ## pre-09/2018 used .lmwithQRP
-    # .lmwithQR() is fast - much faster than qr() and even than chol(crossprod())... 
-    if (EigenDense_QRP_method=="qr") {
+    if (EigenDense_QRP_method==".lmwithQR") {     # .lmwithQR() is fast - much faster than qr() and even than chol(crossprod())... 
+      lmwithqr <- .lmwithQR(sXaug,yy=szAug,returntQ=FALSE,returnR=TRUE) ## using RcppEigen; szAug may be NULL
+      ## we don't request (t) Q from Eigen bc it is comparatively slow 
+      # for (st in names(lmwithqr)) BLOB[[st]] <- lmwithqr[[st]] ## "R_scaled" and optionally "coef"
+      BLOB$R_scaled <- lmwithqr$R_scaled
+      # perm and sortPerm remain NULL
+      if ( ! is.null(szAug)) return(lmwithqr$coef)   
+      # BLOB$sortPerm will be NULL
+    } else if (EigenDense_QRP_method=="qr") {
       ## ( this slightly affects predVar in onelambda vs twolambda)
       blob <- qr(as(sXaug[],"dgCMatrix")) ## blob <- qr(.Rcpp_as_dgCMatrix(sXaug))  ## Matrix::qr
       BLOB$perm <- blob@q + 1L
       BLOB$R_scaled <- as.matrix(qrR(blob,backPermute = FALSE))
-      BLOB$sortPerm <- sort.list(BLOB$perm) 
+      delayedAssign("sortPerm", sort.list(BLOB$perm), assign.env = BLOB ) # never NULL
       if ( ! is.null(szAug)) return(qr.coef(blob,szAug))   
-    } else if (EigenDense_QRP_method==".lmwithQR") { 
-      lmwithqr <- .lmwithQR(sXaug,yy=szAug,returntQ=FALSE,returnR=TRUE) ## using RcppEigen; szAug may be NULL
-      ## we don't request (t) Q from Eigen bc it is comparatively slow 
-      for (st in names(lmwithqr)) BLOB[[st]] <- lmwithqr[[st]] ## "R_scaled" and optionally "coef"
-      # perm and sortPerm remain NULL
-      if ( ! is.null(szAug)) return(BLOB$coef)   
     } else {
       lmwithqrp <- .lmwithQRP(sXaug,yy=szAug,returntQ=FALSE,returnR=TRUE) ## using RcppEigen; szAug may be NULL
       ## we don't request (t) Q from Eigen 
-      for (st in names(lmwithqrp)) BLOB[[st]] <- lmwithqrp[[st]] ## "perm", "R_scaled" and optionally "coef"
+      # for (st in names(lmwithqrp)) BLOB[[st]] <- lmwithqrp[[st]] ## "perm", "R_scaled" and optionally "coef"
+      BLOB$R_scaled <- lmwithqrp$R_scaled
       BLOB$perm <- BLOB$perm +1L
-      BLOB$sortPerm <- sort.list(BLOB$perm) 
-      if ( ! is.null(szAug)) return(BLOB$coef)   
+      delayedAssign("sortPerm", sort.list(BLOB$perm), assign.env = BLOB ) # never NULL
+      if ( ! is.null(szAug)) return(lmwithqrp$coef)   
     }
-    #n_u_h <- attr(sXaug,"n_u_h")
-    #seq_n_u_h <- seq(n_u_h)
+    # any delayedAssign() here would be shadowed by the above return()'s
   } 
   if ( ! is.null(szAug)) {
     rhs <- .crossprodCpp_d(sXaug, szAug)
@@ -52,40 +101,24 @@ def_sXaug_EigenDense_QRP_Chol_scaled <- function(Xaug, # already ZAL_scaled
     # test-adjacency-corrMatrix (R_scaled of dim 114): replicate(100,{source(...)},simplify=FALSE) finds 5e-3s advantage per replicate for backsolve over .Rcpp_backsolve .
     # I.e., fairly NS; and other tests are not more discriminating
     rhs <- backsolve(BLOB$R_scaled, backsolve(BLOB$R_scaled, rhs, transpose = TRUE))
-    if ( is.null(BLOB$sortPerm)) {
+    if ( is.null(BLOB$sortPerm)) { # default
       return(rhs)
     } else return(rhs[BLOB$sortPerm,,drop=FALSE])
   }
   # ELSE
-  if ( is.null(BLOB$R_R_v) && which %in% c("absdiag_R_v","d2hdv2","solve_d2hdv2","hatval_Z","R_scaled_v_h_blob",
-                                           "logdet_R_scaled_v","Mg_invH_g")) {
-    seq_n_u_h <- seq_len(attr(sXaug,"n_u_h"))
-    # remove $u_h_cols_on_left code in version 2.1.61 since there is a bug in it: further code may require 
-    #  no nontrivial perm_R_v, so we remove perm_R_v from further code (as in Matrix_QRP_CHM_scaled version)
-    if ( is.null(BLOB$sortPerm)) {
-      BLOB$R_R_v <- BLOB$R_scaled[seq_n_u_h,seq_n_u_h, drop=FALSE]
-    } else {
-      wd2hdv2w <- .crossprodCpp_d(BLOB$R_scaled[,BLOB$sortPerm[ seq_n_u_h ], drop=FALSE], NULL)
-      BLOB$R_R_v <- .Rcpp_chol_R(wd2hdv2w)$R
-    }
-  }
-  # 
-  if (which=="Qt_leftcols*B") { # use tQ=R^{-T}sX^T in a context where we need only the leftcols of sX^T ie bottom rows of sX
+  if (which=="Qt_leftcols*B") { # use tQ=R^{-T}sX^T in a context where we need only the leftcols of sX^T ie ZX rows of sX
+    # if (.is_evaluated("t_Q_scaled",BLOB)) browser() # then we should surely use it, but this does not happen
     if ( ! is.null(BLOB$perm)) {
-      return(backsolve(BLOB$R_scaled,.crossprod(sXaug[-seq(attr(sXaug,"n_u_h")),BLOB$perm], B),transpose=TRUE))
-    } else return(backsolve(BLOB$R_scaled,.crossprod(sXaug[-seq(attr(sXaug,"n_u_h")),], B),transpose=TRUE))
+      return(backsolve(BLOB$R_scaled,.crossprod(sXaug[-BLOB$seq_n_u_h, BLOB$perm], B),transpose=TRUE))
+    } else return(backsolve(BLOB$R_scaled,.crossprod(sXaug[-BLOB$seq_n_u_h,], B),transpose=TRUE))
   }
+  if (which=="logdet_sqrt_d2hdv2") { return(BLOB$logdet_sqrt_d2hdv2)} 
+  if (which=="logdet_r22") { return(BLOB$logdet_r22) }
   if (which=="hatval_Z") { ## Pdiag; note that these leverages are constant wrt to any diagonal rescaling of the Z block
     if (is.null(BLOB$hatval_Z_)) {
       if (FALSE) { # that is correct when .lmwithQR was used but not .lmwithQRP. 
-        # Further, I would need an u_h_cols_on_the_left instead of BLOB$perm for maximum use of this.  
-        n_u_h <- attr(sXaug,"n_u_h")
-        if ( is.null(BLOB$t_Q_scaled) ) {
-          if ( ! is.null(BLOB$perm)) {
-            BLOB$t_Q_scaled <- backsolve(BLOB$R_scaled,t(sXaug[,BLOB$perm]),transpose=TRUE)
-          } else BLOB$t_Q_scaled <- backsolve(BLOB$R_scaled,t(sXaug),transpose=TRUE)
-        }
-        tmp_t_Qq_scaled <- BLOB$t_Q_scaled[seq_len(n_u_h),]
+        # Further, I would need an u_h_cols_on_left instead of BLOB$perm for maximum use of this.  
+        tmp_t_Qq_scaled <- BLOB$t_Q_scaled[BLOB$seq_n_u_h,]
         tmp_t_Qq_scaled <- tmp_t_Qq_scaled^2
         tmp <- colSums(tmp_t_Qq_scaled)
         phipos <- n_u_h+seq_len(nrow(sXaug)-n_u_h)
@@ -95,57 +128,41 @@ def_sXaug_EigenDense_QRP_Chol_scaled <- function(Xaug, # already ZAL_scaled
       ## As for the sparse version:
       ## t(sXaug[,u_h cols]= (I, scaled t(ZAL)) i.e. is scaled such that the left block is an identity matrix, so we can work 
       ## on two separate blocks if the Cholesky is not permuted. Then 
-      if (is.null(lev_lambda <- BLOB$inv_factor_wd2hdv2w)) {
-        #lev_lambda <- BLOB$inv_factor_wd2hdv2w <- backsolve(BLOB$R_R_v,diag(ncol(BLOB$R_R_v)),transpose=TRUE)
-        # test-spaMM # measurable time gain by .Rcpp_backsolve() possibly bc it's the full inverse which is computed. 
-        lev_lambda <- BLOB$inv_factor_wd2hdv2w <- .Rcpp_backsolve(BLOB$R_R_v, NULL, # i.e., diag(), but without creating it 
-                                                                  #                   (but this does not explain most of the time gain)
-                                                                  transpose=TRUE) 
-      } 
+      #lev_lambda <- BLOB$inv_factor_wd2hdv2w <- backsolve(BLOB$R_R_v,diag(ncol(BLOB$R_R_v)),transpose=TRUE)
+      # test-spaMM # measurable time gain by .Rcpp_backsolve() possibly bc it's the full inverse which is computed. 
+      lev_lambda <- BLOB$inv_factor_wd2hdv2w 
       lev_lambda <- lev_lambda^2
       lev_lambda <- colSums(lev_lambda)
       n_u_h <- attr(sXaug,"n_u_h")
-      phipos <- (n_u_h+1):nrow(sXaug)
-      lev_phi <- .tcrossprod(BLOB$inv_factor_wd2hdv2w, sXaug[phipos, seq_len(n_u_h) ]) # backsolve(BLOB$R_R_v, t(sXaug[phipos, seq_len(n_u_h) ]),transpose=TRUE) ## fixme the t() may still be costly
+      phipos <- (n_u_h+1):nrow(sXaug) #                get the scaled-ZAL block:
+      lev_phi <- .tcrossprod(BLOB$inv_factor_wd2hdv2w, sXaug[phipos, BLOB$seq_n_u_h ]) # backsolve(BLOB$R_R_v, t(sXaug[phipos, seq_len(n_u_h) ]),transpose=TRUE) 
       lev_phi <- lev_phi^2
       lev_phi <- colSums(lev_phi)
-      BLOB$hatval_Z_ <-  list(lev_lambda=lev_lambda,lev_phi=lev_phi)
+      BLOB$hatval_Z_ <-  list(lev_lambda=lev_lambda,lev_phi=lev_phi) # __F I X M E__ could separate comput of the two elements ? when is that useful?
     }
     return(BLOB$hatval_Z_)
   } 
   if (which=="solve_d2hdv2") { ## R'R[seq_n_u_h, seq_n_u_h] gives -d2hd_scaled_v2.
     if ( is.null(B) ) {
-      if ( is.null(BLOB$inv_d2hdv2)) {
-        if (is.null(BLOB$invsqrtwranef)) BLOB$invsqrtwranef <- 1/sqrt(attr(sXaug,"w.ranef")) 
-        if (is.null(BLOB$inv_factor_wd2hdv2w)) {
-          inv_d2hdv2 <- chol2inv(BLOB$R_R_v)
-        } else inv_d2hdv2 <- .crossprod(BLOB$inv_factor_wd2hdv2w) # i.e., crossfactor
-        inv_d2hdv2 <- .m_Matrix_times_Dvec(inv_d2hdv2, BLOB$invsqrtwranef) ## _m_atrix sweep 2L
-        BLOB$inv_d2hdv2 <- - .Dvec_times_matrix(BLOB$invsqrtwranef,inv_d2hdv2)
-      }
       return(BLOB$inv_d2hdv2)
     } else { ## solve (matrix,y)
-      if (is.null(BLOB$inv_d2hdv2)) {
-        if (is.null(BLOB$invsqrtwranef)) BLOB$invsqrtwranef <- 1/sqrt(attr(sXaug,"w.ranef")) 
+      if (.is_evaluated("inv_d2hdv2", BLOB)) {
+        return(BLOB$inv_d2hdv2 %*% B) ## inv_d2hdv2 tends to be dense
+      } else {
         if (is.matrix(B)) {
           rhs <- .Dvec_times_matrix(BLOB$invsqrtwranef,B)
         } else rhs <- BLOB$invsqrtwranef * B
-        if (is.null(BLOB$inv_factor_wd2hdv2w)) {
+        if (.is_evaluated("inv_factor_wd2hdv2w", BLOB)) {
           # test-spaMM Nugget multinomial inverse-Gamma .... no clear benefit in using .Rcpp_chol2solve()
-          rhs <- backsolve(BLOB$R_R_v, backsolve(BLOB$R_R_v, rhs, transpose = TRUE))
-        } else rhs <- .crossprod(BLOB$inv_factor_wd2hdv2w, drop(BLOB$inv_factor_wd2hdv2w %*% rhs)) # typical sscaled computation
+          rhs <- .crossprod(BLOB$inv_factor_wd2hdv2w, drop(BLOB$inv_factor_wd2hdv2w %*% rhs)) # typical sscaled computation
+        } else rhs <- backsolve(BLOB$R_R_v, backsolve(BLOB$R_R_v, rhs, transpose = TRUE))
         if (is.matrix(rhs)) {
           rhs <- .Dvec_times_matrix(BLOB$invsqrtwranef,rhs)
         } else rhs <- BLOB$invsqrtwranef * rhs
         return( - rhs)
-      } else {
-        return(BLOB$inv_d2hdv2 %*% B) ## inv_d2hdv2 tends to be dense
       }
     }
   } 
-  if (which=="R_scaled") { ## for logdet_r22
-    return(BLOB$R_scaled)
-  } 
   if (which=="R_scaled_blob") {
     if (is.null(BLOB$R_scaled_blob)) {
       if ( ! is.null(BLOB$sortPerm)) {
@@ -166,9 +183,7 @@ def_sXaug_EigenDense_QRP_Chol_scaled <- function(Xaug, # already ZAL_scaled
   } 
   if (which=="R_beta_blob") {
     if (is.null(BLOB$R_beta_blob)) {
-      n_u_h <- attr(sXaug,"n_u_h")
-      seq_n_u_h <- seq(n_u_h)
-      X <- as.matrix(sXaug[-seq_n_u_h,-seq_n_u_h]) ## following code assuming it is dense...
+      X <- as.matrix(sXaug[-BLOB$seq_n_u_h,-BLOB$seq_n_u_h]) ## following code assuming it is dense...
       R_beta <- .lmwithQR(X,yy=NULL,returntQ=FALSE,returnR=TRUE)$R_scaled
       diag_pRtRp_beta <-  colSums(R_beta^2)
       BLOB$R_beta_blob <- list(R_beta=R_beta,diag_pRtRp_beta=diag_pRtRp_beta, 
@@ -177,50 +192,30 @@ def_sXaug_EigenDense_QRP_Chol_scaled <- function(Xaug, # already ZAL_scaled
     }
     return(BLOB$R_beta_blob)
   } 
-  if (which=="hatval") {
-    if ( is.null(BLOB$t_Q_scaled) ) {
-      if ( ! is.null(BLOB$perm)) {
-        BLOB$t_Q_scaled <- backsolve(BLOB$R_scaled,t(sXaug[,BLOB$perm]),transpose=TRUE)
-      } else BLOB$t_Q_scaled <- backsolve(BLOB$R_scaled,t(sXaug),transpose=TRUE)
-    }
-    return(colSums(BLOB$t_Q_scaled^2))
-  } 
+  if (which=="hatval") {return(BLOB$hatval) }  ##  REML hatval computation (also named hatval_ZX)
   if (which=="Mg_invH_g") { # - sum(B %*% inv_d2hdv2 %*% B) # was oddly inconsistent with the Matrix_QRP_CHM code until change in v3.6.4
-    if (is.null(BLOB$invsqrtwranef)) BLOB$invsqrtwranef <- 1/sqrt(attr(sXaug,"w.ranef")) 
     rhs <- BLOB$invsqrtwranef * B
-    if (is.null(BLOB$inv_factor_wd2hdv2w)) {
-      rhs <- .Rcpp_backsolve(BLOB$R_R_v,rhs,transpose=TRUE)
-    } else rhs <- BLOB$inv_factor_wd2hdv2w %*% rhs
+    if (.is_evaluated("inv_factor_wd2hdv2w", BLOB)) {
+      rhs <- BLOB$inv_factor_wd2hdv2w %*% rhs
+    } else rhs <- .Rcpp_backsolve(BLOB$R_R_v,rhs,transpose=TRUE)
     return(sum(rhs^2))
   } 
   if (which=="Mg_solve_g") {
-    if (is.null(BLOB$invsqrtwranef)) BLOB$invsqrtwranef <- 1/sqrt(attr(sXaug,"w.ranef")) 
-    seq_n_u_h <- seq_len(attr(sXaug,"n_u_h"))
     rhs <- B
-    rhs[seq_n_u_h] <- BLOB$invsqrtwranef * rhs[seq_n_u_h]
+    rhs[BLOB$seq_n_u_h] <- BLOB$invsqrtwranef * rhs[BLOB$seq_n_u_h]
     if (is.null(BLOB$perm)) {
       rhs <- backsolve(BLOB$R_scaled, rhs, transpose = TRUE)
     } else rhs <- backsolve(BLOB$R_scaled, rhs[BLOB$perm], transpose = TRUE)
     return(sum(rhs^2))
   } 
   if (which=="Mg_invXtWX_g") { ## 
-    if (is.null(BLOB$XtWX)) BLOB$XtWX <- sXaug[-seq_n_u_h,-seq_n_u_h]
+    if (is.null(BLOB$XtWX)) BLOB$XtWX <- .crossprod(sXaug[-BLOB$seq_n_u_h,-BLOB$seq_n_u_h])
     Mg_invXtWX_g <- crossprod(B,solve(BLOB$XtWX,B))[1L] # [1L] drops possible Matrix class...
     return(Mg_invXtWX_g)
   } 
-  if (which %in% c("logdet_R_scaled_b_v")) {
-    if (is.null(BLOB$logdet_R_scaled_b_v)) BLOB$logdet_R_scaled_b_v <- sum(log(abs(.diagfast(x=BLOB$R_scaled))))
-    return(BLOB$logdet_R_scaled_b_v)
-  } 
-  if (which %in% c("logdet_R_scaled_v")) {
-    if (is.null(BLOB$logdet_R_scaled_v)) {
-      if (is.null(BLOB$absdiag_R_v)) BLOB$absdiag_R_v <- abs(diag(x=BLOB$R_R_v)) ## as in "absdiag_R_v"
-      BLOB$logdet_R_scaled_v <- sum(log(BLOB$absdiag_R_v)) 
-    }
-    return(BLOB$logdet_R_scaled_v)
-  } 
+  if (which=="logdet_R_scaled_b_v") {return(BLOB$logdet_R_scaled_b_v)} 
   if (which=="beta_cov_info_from_sXaug") {  
-    return(.calc_beta_cov_info_from_sXaug(BLOB=BLOB, sXaug=sXaug))
+    return(.calc_beta_cov_info_from_sXaug(BLOB=BLOB, sXaug=sXaug, tcrossfac=solve(BLOB$R_scaled)))
   } 
   if (which=="beta_cov_info_from_wAugX") { ## using a weighted Henderson's augmented design matrix, not a true sXaug  
     if (TRUE) {
@@ -245,17 +240,11 @@ def_sXaug_EigenDense_QRP_Chol_scaled <- function(Xaug, # already ZAL_scaled
     # otherwise, dropO(,tol=...), + some fix in summary.HLfit for matrix[] <- Matrix assignment, would be useful.  
     return(beta_v_cov)
   } else if (which %in% c("absdiag_R_v")) { 
-    if (is.null(BLOB$absdiag_R_v)) BLOB$absdiag_R_v <- abs(diag(x=BLOB$R_R_v)) 
     return(BLOB$absdiag_R_v)
   # } else if (which=="sortPerm") { 
   #   return(BLOB$sortPerm)
   }
   if (which=="t_Q_scaled") { # residual call for non-standard REML 
-    if ( is.null(BLOB$t_Q_scaled) ) {
-      if ( ! is.null(BLOB$perm)) {
-        BLOB$t_Q_scaled <- backsolve(BLOB$R_scaled,t(sXaug[,BLOB$perm]),transpose=TRUE)
-      } else BLOB$t_Q_scaled <- backsolve(BLOB$R_scaled,t(sXaug),transpose=TRUE)
-    }
     return(BLOB$t_Q_scaled)
   }
   if (which=="d2hdv2") {
@@ -273,16 +262,6 @@ def_sXaug_EigenDense_QRP_Chol_scaled <- function(Xaug, # already ZAL_scaled
 get_from_MME.sXaug_EigenDense_QRP_Chol_scaled <- function(sXaug,which="",szAug=NULL,B=NULL,
                                     damping, LMrhs, ...) {
   resu <- switch(which,
-                 "logdet_sqrt_d2hdv2" = {
-                   w.ranef <- attr(sXaug,"w.ranef")
-                   absdiag_R_v <- .sXaug_EigenDense_QRP_Chol_scaled(sXaug,which="absdiag_R_v")
-                   sum(log(sqrt(w.ranef) * absdiag_R_v))
-                 },
-                 "logdet_r22" = { # the R's are H-scaled but r22 is H-unscaled... messy ! f i x m e
-                   logdet_R_scaled_b_v <- .sXaug_EigenDense_QRP_Chol_scaled(sXaug,which="logdet_R_scaled_b_v")
-                   logdet_R_scaled_v <- .sXaug_EigenDense_QRP_Chol_scaled(sXaug,which="logdet_R_scaled_v")
-                   logdet_R_scaled_b_v - logdet_R_scaled_v - attr(sXaug,"pforpv")*log(attr(sXaug,"H_global_scale"))/2 
-                 },
                  "LevMar_step" = {
                    R_scaled_blob <- .sXaug_EigenDense_QRP_Chol_scaled(sXaug,which="R_scaled_blob")
                    dampDpD <- damping*R_scaled_blob$diag_pRtRp ## NocedalW p. 266
