@@ -11,4 +11,9 @@ p1 <- predict(fitobject)[c(4,3),]
 p2 <- predict(fitobject, newdata=rbind(clinics[c(4,3),3:4],c(1,9)))[1:2,] 
 ## The newdata have a >1-subset (not one, not all) of the original levels of the 'clinic' factor.
 ## For un-correlated random effects, earlier versions returned an incorrect result in this case.
-testthat::expect_true(diff(range(p1-p2))<1e-12)
+crit <- diff(range(p1-p2))
+if (spaMM.getOption("fpot_tol")>0) {
+  testthat::test_that(paste0("test-more-predict: criterion was ",signif(crit,4)," >1e-12"), testthat::expect_true(crit<1e-12)) 
+} else testthat::expect_true(crit<1e-12)
+
+                    
